  /*
  This is an example file for using the time function in ESP8266 or ESP32 tu get NTP time
  It offers two functions:

  - getNTPtime(struct tm * info, uint32_t ms) where info is a structure which contains time
  information and ms is the time the service waits till it gets a response from NTP.
  Each time you cann this function it calls NTP over the net.

  If you do not want to call an NTP service every second, you can use
  - getTimeReducedTraffic(int ms) where ms is the the time between two physical NTP server calls. Betwwn these calls,
  the time structure is updated with the (inaccurate) timer. If you call NTP every few minutes you should be ok

  The time structure is called tm and has teh following values:

  Definition of struct tm:
  Member  Type  Meaning Range
  tm_sec  int seconds after the minute  0-61*
  tm_min  int minutes after the hour  0-59
  tm_hour int hours since midnight  0-23
  tm_mday int day of the month  1-31
  tm_mon  int months since January  0-11
  tm_year int years since 1900
  tm_wday int days since Sunday 0-6
  tm_yday int days since January 1  0-365
  tm_isdst  int Daylight Saving Time flag

  because the values are somhow akwardly defined, I introduce a function makeHumanreadable() where all values are adjusted according normal numbering.
  e.g. January is month 1 and not 0 And Sunday or monday is weekday 1 not 0 (according definition of MONDAYFIRST)

  Showtime is an example on how you can use the time in your sketch

  The functions are inspired by work of G6EJD ( https://www.youtube.com/channel/UCgtlqH_lkMdIa4jZLItcsTg )

*/

#ifdef ESP8266
#include <ESP8266WiFi.h>
#else
#include <WiFi.h>
#endif
#include <time.h>

#include <Adafruit_GFX.h>
#include <Adafruit_NeoMatrix.h>
#include <Adafruit_NeoPixel.h>

#define PIN 10 //Din on pin SD3 of arduino

Adafruit_NeoMatrix matrix = Adafruit_NeoMatrix(32, 8, PIN,  NEO_MATRIX_TOP + NEO_MATRIX_LEFT + NEO_MATRIX_COLUMNS + NEO_MATRIX_ZIGZAG, NEO_GRB + NEO_KHZ800); //Matrix 8*32

const uint16_t colors[] = {
  matrix.Color(255, 0, 0), matrix.Color(0, 255, 0), matrix.Color(0, 0, 255) };//colors

const char* ssid = "XXXXXXXXXXXXXX"; //Change this
const char* password = "XXXXXXXXXXXX"; //Change this


const char* NTP_SERVER = "pool.ntp.org";
const char* TZ_INFO    = "CET-1CEST-2,M3.5.0/02:00:00,M10.5.0/03:00:00";  //This time zone is for France. Change your time zone (https://remotemonitoringsystems.ca/time-zone-abbreviations.php)

tm timeinfo;
time_t now;
long unsigned lastNTPtime;
unsigned long lastEntryTime;


void setup() {
  Serial.begin(115200);
  Serial.println("\n\nNTP Time Test\n");
  WiFi.begin(ssid, password);

  int counter = 0;
  while (WiFi.status() != WL_CONNECTED) {
    delay(200);
    if (++counter > 100) ESP.restart();
    Serial.print ( "." );
  }
  Serial.println("\n\nWiFi connected\n\n");

  configTime(0, 0, NTP_SERVER);
  setenv("TZ", TZ_INFO, 1);

  if (getNTPtime(10)) {  // wait up to 10sec to sync
  } else {
    Serial.println("Time not set");
    ESP.restart();
  }
  lastNTPtime = time(&now);
  lastEntryTime = millis();

  matrix.begin();
  matrix.setBrightness(5);//0 to 255 (255 maximum)
}


void loop() {
  getNTPtime(10); //refresh every hours
  matrix.fillScreen(0);
  Line_hours(timeinfo.tm_hour,0); //hours,color
  Line_minute(timeinfo.tm_min,1); //minutes,color
  Line_second(timeinfo.tm_sec,2); //second,color
  //Dot_second(timeinfo.tm_sec,2); //Dot
  //Random_second(timeinfo.tm_sec,50); //Random(second,min_random)
  //Fade_second(timeinfo.tm_sec); // Fade
  matrix.show();
  delay(1000); // each second
}

bool getNTPtime(int sec) {

  {
    uint32_t start = millis();
    do {
      time(&now);
      localtime_r(&now, &timeinfo);
      delay(10);
    } while (((millis() - start) <= (1000 * sec)) && (timeinfo.tm_year < (2016 - 1900)));
    if (timeinfo.tm_year <= (2016 - 1900)) return false;  // the NTP call was not successful
    char time_output[30];
    strftime(time_output, 30, "%a  %d-%m-%y %T", localtime(&now));
  }
  return true;
}

void Line_hours(int h, int c)//Line X=0 and X=1
{
  //Show nothing when h=0
  if (h == 0){
  }
  else if (h < 13){
    matrix.drawLine(0, 0, h-1, 0, colors[c]); // (Start X,Y, End X,Y,color)
  }
  else{
    int r = h-12;
    matrix.drawLine(0, 0, 12-1, 0, colors[c]);
    matrix.drawLine(0, 1, r-1, 1, colors[c]);//start with 0 not 1
  }
}
void Line_minute(int m, int c)//Line X=3 and X=4
{
  //Show nothing when m=0
  if (m == 0){
  }
  else if (m < 31){
    matrix.drawLine(0, 3, m-1, 3, colors[c]); // (Start X,Y, End X,Y,color)
  }
  else{
    int r = m-30;
    matrix.drawLine(0, 3, 30-1, 3, colors[c]);
    matrix.drawLine(0, 4, r-1, 4, colors[c]);//start with 0 not 1
  }
}
void Line_second(int s, int c)//Line X=6 and X=7
{
  //Show nothing when s=0
  if (s == 0){
  }
  else if (s < 31){
    matrix.drawLine(0, 6, s-1, 6, colors[c]); // (Start X,Y, End X,Y,color)
  }
  else{
    int r = s-30;
    matrix.drawLine(0, 6, 30-1, 6, colors[c]);
    matrix.drawLine(0, 7, r-1, 7, colors[c]);//start with 0 not 1
  }
}
void Dot_second(int s, int c)//Line X=6 and X=7
{
  //Show nothing when s=0
  if (s < 30){
    matrix.drawPixel(s, 6, colors[c]); 
  }
  else{
    int r = s-30;
   matrix.drawPixel(r, 7, colors[c]);
  }
}
void Random_second(int s,int min_rd)//Line X=6 and X=7
{
  for (uint8_t x=0; x <= s ;x++)
  {
    matrix.drawPixel(constrain(x,0,29), 6, matrix.Color(random(min_rd,254), random(min_rd,254), random(min_rd,254)));
    matrix.drawPixel(x-30, 7, matrix.Color(random(min_rd,254), random(min_rd,254), random(min_rd,254)));
  }
}
void Fade_second(int s)//Line X=6 and X=7
{
  for (uint8_t x=0; x <= s ;x++)
  {
    matrix.drawPixel(constrain(x,0,29), 6, matrix.Color(135+x*0,135+x*1,135+x*3));
    matrix.drawPixel(x-30, 7, matrix.Color(135+x*0,135+x*1,135+x*3));
  }
}
